searchData={"items":[{"type":"module","title":"CCPrecompiler","doc":"Precompile with existing crosscompiler in the system.","ref":"CCPrecompiler.html"},{"type":"function","title":"CCPrecompiler.current_target/1","doc":"","ref":"CCPrecompiler.html#current_target/1"},{"type":"behaviour","title":"CCPrecompiler.CompilationScript","doc":"","ref":"CCPrecompiler.CompilationScript.html"},{"type":"callback","title":"CCPrecompiler.CompilationScript.compile/6","doc":"","ref":"CCPrecompiler.CompilationScript.html#c:compile/6"},{"type":"module","title":"CCPrecompiler.UniversalBinary","doc":"Build a universal binary on macOS","ref":"CCPrecompiler.UniversalBinary.html"},{"type":"module","title":"Example - CCPrecompiler.UniversalBinary","doc":"\"macos-universal\" => {\n    :script, \"\", {CCPrecompiler.UniversalBinary, []}\n  }","ref":"CCPrecompiler.UniversalBinary.html#module-example"},{"type":"extras","title":"Precompilation guide","doc":"# Precompilation guide\n\nThis guide has two sections, the first one is intended for precompiler module developers. It covers a minimal example of creating a precompiler module. The second section is intended for library developers who want their library to be able to use precompiled artefacts in a simple way.","ref":"precompilation_guide.html"},{"type":"extras","title":"Library Developer - Precompilation guide","doc":"This guide assumes you have already added `elixir_make` to your library and you have written a `Makefile` that compiles the native code in your project. Once your native code compile and works as expected, you are now ready to precompile it.\n\nA full demo project is available on [cocoa-xu/cc_precompiler_example](https://github.com/cocoa-xu/cc_precompiler_example).\n\n#","ref":"precompilation_guide.html#library-developer"},{"type":"extras","title":"Setup mix.exs - Precompilation guide","doc":"To use a precompiler module such as the `CCPrecompiler` example above, we first add the precompiler (`:cc_precompiler` here) and `:elixir_make` to `deps`.\n\n```elixir\ndef deps do\n[\n    # ...\n    {:elixir_make, \"~> 0.6\", runtime: false},\n    {:cc_precompiler, \"~> 0.1\", runtime: false, github: \"cocoa-xu/cc_precompiler\"}\n    # ...\n]\nend\n```\n\nThen add `:elixir_make` to the `compilers` list, and set `CCPrecompile` as the value for `make_precompiler`.\n\n```elixir\n@version \"0.1.0\"\ndef project do\n  [\n    # ...\n    compilers: [:elixir_make] ++ Mix.compilers(),\n    # elixir_make specific config\n    make_precompiler: {:nif, CCPrecompiler},\n    make_precompiler_url: \"https://github.com/cocoa-xu/cc_precompiler_example/releases/download/v#{@version}/@{artefact_filename}\",\n    make_precompiler_filename: \"nif\",\n    make_precompiler_priv_paths: [\"nif.*\"],\n    make_precompiler_unavailable_target: :compile,\n    # ...\n  ]\nend\n```\n\nAnother required field is `make_precompiled_url`. It is a URL template to the artefact file.\n\n`@{artefact_filename}` in the URL template string will be replaced by corresponding artefact filenames when fetching them. For example, `cc_precompiler_example-nif-2.16-x86_64-linux-gnu-0.1.0.tar.gz`.\n\nNote that there is an optional config key for elixir_make, `make_precompiler_filename`. If the name (file extension does not count) of the shared library is different from your app's name, then `make_precompiler_filename` should be set. For example, if the app name is `\"cc_precompiler_example\"` while the name shared library is `\"nif.so\"` (or `\"nif.dll\"` on windows), then `make_precompiler_filename` should be set as `\"nif\"`.\n\nAnother optional config key is `make_precompiler_priv_paths`. For example, say the `priv` directory is organised as follows in Linux, macOS and Windows respectively,\n\nAlso, you can specify how to recover from unavailable targets using the `make_precompiler_unavailable_target` config key. Allowed values are `:compile` and `:ignore`. Defaults to `:compile`.\n\nIt is also possible to pass in a 2-arity function to `make_precompiler_unavailable_target`: the first argument is the triplet of the unavailable target, and the second argument is a list that contains all available targets given by the precompiler.\n\n```\n# Linux\n.\n├── assets\n│   ├── model.onnx\n│   └── data.json\n├── lib\n│   ├── libpriv1.so\n│   ├── libpriv2.so\n│   └── libpriv3.so\n└── nif.so\n\n# macOS\n.\n├── assets\n│   ├── model.onnx\n│   └── data.json\n├── lib\n│   ├── libpriv1.dylib\n│   ├── libpriv2.dylib\n│   └── libpriv3.dylib\n└── nif.so\n\n# Windows\n.\n├── assets\n│   ├── model.onnx\n│   └── data.json\n├── lib\n│   ├── libpriv1.dll\n│   ├── libpriv2.dll\n│   └── libpriv3.dll\n└── nif.dll\n```\n\nBy default, everything in `priv` will be included in the precompiled tar file. However, files in `assets` can be very large or platform-independent, therefore, we would like to only include the `nif.so` (`nif.dll`) file and everything in the `lib` directory in the precompiled tar file to reduce the footprint. In this case, we can set `make_precompiler_priv_paths` to `[\"nif.so\", \"nif.dll\", \"lib\"]`.\n\nOf course, wildcards (`?`, `**`, `*`) are supported when specifying files. For example, `[\"nif.*\", \"lib/*.so\", \"lib/*.dll\", \"lib/*.dylib\"]` will include `nif.so` (Linux/macOS) or `nif.dll` (Windows), and `.so` or `.dll` files in the `lib` directory. \n\nDirectory structures and symbolic links are preserved.\n\n### (Optional) Test the NIF code locally\n\nTo test the NIF code locally, you can either set `force_build` to `true` or append `\"-dev\"` to your NIF library's version string.\n\n```elixir\n@version \"0.1.0-dev\"\n\ndef project do\n  [\n    # either append `\"-dev\"` to your NIF library's version string\n    version: @version,\n    # or set force_build to true\n    force_build: true,\n    # ...\n  ]\nend\n```\n\nDoing so will ask `elixir_make` to only compile for the current host instead of building for all available targets.\n\n```shell\n$ mix compile\ncc -shared -std=c11 -O3 -fPIC -I\"/usr/local/lib/erlang/erts-13.0.3/include\" -undefined dynamic_lookup -flat_namespace -undefined suppress \"/Users/cocoa/git/cc_precompiler_example/c_src/cc_precompiler_example.c\" -o \"/Users/cocoa/Git/cc_precompiler_example/_build/dev/lib/cc_precompiler_example/priv/nif.so\"\n$ mix test\nmake: Nothing to be done for `build'.\nGenerated cc_precompiler_example app\n.\n\nFinished in 0.00 seconds (0.00s async, 0.00s sync)\n1 test, 0 failures\n\nRandomized with seed 102464\n```\n\n#","ref":"precompilation_guide.html#setup-mix-exs"},{"type":"extras","title":"Precompile for available targets - Precompilation guide","doc":"It's possible to either setup a CI task to do the precompilation job or precompile on a local machine and upload the precompiled artefacts.\n\nTo precompile for all targets on a local machine:\n\n```shell\nMIX_ENV=prod mix elixir_make.precompile\n```\n\nEnvironment variable `ELIXIR_MAKE_CACHE_DIR` can be used to set the cache dir for the precompiled artefacts, for instance, to output precompiled artefacts in the cache directory of the current working directory, `export ELIXIR_MAKE_CACHE_DIR=\"$(pwd)/cache\"`.\n\nTo setup a CI task such as GitHub Actions, the following workflow file can be used for reference:\n\n```yml\nname: precompile\n\non:\n  push:\n    tags:\n      - 'v*'\n\njobs:\n  linux:\n    runs-on: ubuntu-latest\n    env:\n      MIX_ENV: \"prod\"\n    steps:\n      - uses: actions/checkout@v3\n\n      - uses: erlef/setup-beam@v1\n        with:\n          otp-version: \"25.1\"\n          elixir-version: \"1.14\"\n\n      - name: Install system dependencies\n        run: |\n          sudo apt-get update\n          sudo apt-get install -y build-essential automake autoconf pkg-config bc m4 unzip zip \\\n            gcc g++ \\\n            gcc-i686-linux-gnu g++-i686-linux-gnu \\\n            gcc-aarch64-linux-gnu g++-aarch64-linux-gnu \\\n            gcc-arm-linux-gnueabihf g++-arm-linux-gnueabihf \\\n            gcc-riscv64-linux-gnu g++-riscv64-linux-gnu \\\n            gcc-powerpc64le-linux-gnu g++-powerpc64le-linux-gnu \\\n            gcc-s390x-linux-gnu g++-s390x-linux-gnu\n\n      - name: Get musl cross-compilers (Optional, use this if you have musl targets to compile)\n        run: |\n          for musl_arch in x86_64 aarch64 riscv64\n          do\n            wget \"https://musl.cc/${musl_arch}-linux-musl-cross.tgz\" -O \"${musl_arch}-linux-musl-cross.tgz\"\n            tar -xf \"${musl_arch}-linux-musl-cross.tgz\"\n          done\n\n      - name: Mix Test\n        run: |\n          # Optional, use this if you have musl targets to compile\n          for musl_arch in x86_64 aarch64 riscv64\n          do\n            export PATH=\"$(pwd)/${musl_arch}-linux-musl-cross/bin:${PATH}\"\n          done\n\n          mix deps.get\n          MIX_ENV=test mix test\n\n      - name: Create precompiled library\n        run: |\n          export ELIXIR_MAKE_CACHE_DIR=$(pwd)/cache\n          mkdir -p \"${ELIXIR_MAKE_CACHE_DIR}\"\n          mix elixir_make.precompile\n\n      - uses: softprops/action-gh-release@v1\n        if: startsWith(github.ref, 'refs/tags/')\n        with:\n          files: |\n            cache/*.tar.gz\n\n  macos:\n    runs-on: macos-11\n    env:\n      MIX_ENV: \"prod\"\n\n    steps:\n      - uses: actions/checkout@v3\n\n      - name: Install erlang and elixir\n        run: |\n          brew install erlang elixir\n          mix local.hex --force\n          mix local.rebar --force\n\n      - name: Mix Test\n        run: |\n          mix deps.get\n          MIX_ENV=test mix test\n\n      - name: Create precompiled library\n        run: |\n          export ELIXIR_MAKE_CACHE_DIR=$(pwd)/cache\n          mkdir -p \"${ELIXIR_MAKE_CACHE_DIR}\"\n          mix elixir_make.precompile\n\n      - uses: softprops/action-gh-release@v1\n        if: startsWith(github.ref, 'refs/tags/')\n        with:\n          files: |\n            cache/*.tar.gz\n```\n\n#","ref":"precompilation_guide.html#precompile-for-available-targets"},{"type":"extras","title":"Generate checksum file - Precompilation guide","doc":"After CI has finished, you can fetch the precompiled binaries from GitHub.\n\n```shell\n$ MIX_ENV=prod mix elixir_make.checksum --all --ignore-unavailable\n```\n\nMeanwhile, a checksum file will be generated. In this example, the checksum file will be named as `checksum.exs` in current working directory.\n\nThis checksum file is extremely important in the scenario where you need to release a Hex package using precompiled NIFs. It's **MANDATORY** to include this file in your Hex package (by updating the `files` field in the `mix.exs`). Otherwise your package **won't work**.\n\n```elixir\ndefp package do\n  [\n    files: [\n      # ...\n      \"checksum.exs\",\n      # ...\n    ],\n    # ...\n  ]\nend\n```\n\nHowever, there is no need to track the checksum file in your version control system (git or other).\n\n### (Optional) Test fetched artefacts can work locally\n```shell\n# delete previously built binaries so that\n# elixir_make will try to restore the NIF library\n# from the downloaded tarball file\n$ rm -rf _build/prod/lib/cc_precompiler_example\n# set to prod env and test everything\n$ MIX_ENV=prod mix test\n==> castore\nCompiling 1 file (.ex)\nGenerated castore app\n==> elixir_make\nCompiling 5 files (.ex)\nGenerated elixir_make app\n==> cc_precompiler\nCompiling 1 file (.ex)\nGenerated cc_precompiler app\n\n20:47:42.262 [debug] Restore NIF for current node from: /Users/cocoa/Library/Caches/cc_precompiler_example-nif-2.16-aarch64-apple-darwin-0.1.0.tar.gz\n==> cc_precompiler_example\nCompiling 1 file (.ex)\nGenerated cc_precompiler_example app\n.\n\nFinished in 0.01 seconds (0.00s async, 0.01s sync)\n1 test, 0 failures\n\nRandomized with seed 539590\n```","ref":"precompilation_guide.html#generate-checksum-file"},{"type":"extras","title":"Recommended flow - Precompilation guide","doc":"To recap, the suggested flow is the following:\n\n1. Choose an appropriate precompiler for your NIF library and set all necessary options in the `mix.exs`.\n2. (Optional) Test if your NIF library compiles locally.\n\n  ```shell\n  mix compile\n  mix test\n  ```\n\n3. (Optional) Test if your NIF library can precompile to all specified targets locally.\n  ```shell\n  MIX_ENV=prod mix elixir_make.precompile\n  ```\n\n4. Precompile your library on CI or locally.\n\n  ```shell\n  # locally\n  MIX_ENV=prod mix elixir_make.precompile\n  # CI\n  # please see the docs above\n  ```\n\n5. Fetch precompiled binaries from GitHub.\n\n  ```shell\n  # only fetch artefact for current host\n  MIX_ENV=prod mix elixir_make.checksum --only-local --print\n  # fetch all\n  MIX_ENV=prod mix elixir_make.checksum --all --print\n  # to fetch all available artefacts at the moment\n  MIX_ENV=prod mix elixir_make.checksum --all --print --ignore-unavailable\n  ```\n\n6. (Optional) Test if the downloaded artefacts works as expected.\n\n  ```shell\n  rm -rf _build/prod/lib/NIF_LIBRARY_NAME\n  MIX_ENV=prod mix test\n  ```\n\n6. Update Hex package to include the checksum file.\n7. Release the package to Hex.pm (make sure your release includes the correct files).","ref":"precompilation_guide.html#recommended-flow"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.1.9 (2023-11-16) - Changelog","doc":"- added an `exclude_current_target` option\n- updated ex_doc","ref":"changelog.html#v0-1-9-2023-11-16"},{"type":"extras","title":"v0.1.8 (2023-07-19) - Changelog","doc":"#","ref":"changelog.html#v0-1-8-2023-07-19"},{"type":"extras","title":"Changed - Changelog","doc":"Fixed `CCPrecompiler.all_supported_targets(:fetch)`. It should fetch and merge default available compilers when `include_default_ones` is `true`.","ref":"changelog.html#changed"},{"type":"extras","title":"v0.1.7 (2022-03-13) - Changelog","doc":"#","ref":"changelog.html#v0-1-7-2022-03-13"},{"type":"extras","title":"Added - Changelog","doc":"- using `:include_default_ones` in `project.cc_precompiler.compilers`. Default (cross-)compiler will be included if it's `true`, otherwise only specified targets will be used.\n\nDefault value of `:include_default_ones` is `false` to avoid breaking changes.\n\nIf a custom target has the same name as a default one, then the custom one will override the default configuration for that target (e.g., the `x86_64-linux-gnu` entry below will override the default gcc configuration and use clang instead).\n\n```elixir\ndef project do\n  [\n    # ...\n    cc_precompiler: [\n      compilers: %{\n        {:unix, :linux} => %{\n          :include_default_ones => true,\n          \"my-custom-target\" => {\n            \"my-custom-target-gcc\",\n            \"my-custom-target-g++\"\n          },\n          \"x86_64-linux-gnu\" => {\n            \"x86_64-linux-gnu-clang\",\n            \"x86_64-linux-gnu-clang++\"\n          }\n        }\n      }\n    ]\n  ]\nend\n```","ref":"changelog.html#added"},{"type":"extras","title":"v0.1.6 (2022-02-20) - Changelog","doc":"#","ref":"changelog.html#v0-1-6-2022-02-20"},{"type":"extras","title":"Added - Changelog","doc":"- allow missing CC or CXX when detecting available targets by setting `allow_missing_compiler` to `true`.\n\n  Adding this option because there is no need to require the presence of both CC and CXX for projects that only uses one of them.\n\n  ```elixir\n  def project do\n    [ \n    # ...\n    cc_precompiler: [\n        # optional config key\n        #   true - the corresponding target will be available as long as we can detect either `CC` or `CXX`\n        #   false  - both `CC` and `CXX` should be present on the system\n        # defaults to `false`\n        allow_missing_compiler: false,\n        # ...\n    ],\n    # ...\n    ]\n  end\n  ```","ref":"changelog.html#added"}],"content_type":"text/markdown"}